# gui.py
# Yu Gui    01/12/2017
# Isopass
# NYU CIMS

from . import hash


def run_gui():
    import tkinter
    from tkinter import messagebox

    # generate password by calling hash module
    # show password in a info message box
    def generate_pwd(*args):
        str_app_pwd = app_pwd.get()
        str_master_pwd = master_pwd.get()
        str_result = hash.run_hash(str_app_pwd, str_master_pwd)
        messagebox.showinfo('IsoPass', str_result)
        return ()

    # create root window instance
    root = tkinter.Tk()
    root.title('IsoPass')

    # declare entry vars
    app_pwd = tkinter.StringVar()
    master_pwd = tkinter.StringVar()

    # declare layout elements
    label_slogan = tkinter.Label(root, text='IsoPass')
    label_app_pwd = tkinter.Label(root, text='Application String')
    entry_app_pwd = tkinter.Entry(root, textvariable=app_pwd)
    label_master_pwd = tkinter.Label(root, text='Master Pwd')
    entry_master_pwd = tkinter.Entry(root, textvariable=master_pwd)
    button_generate = tkinter.Button(root, text='generate', command=generate_pwd)

    # place elements on the root window
    label_slogan.grid(row=0, padx=10, pady=20)
    label_app_pwd.grid(row=1, padx=10, pady=10)
    entry_app_pwd.grid(row=2, padx=10, pady=0)
    label_master_pwd.grid(row=3, padx=10, pady=10)
    entry_master_pwd.grid(row=4, padx=10, pady=0)
    button_generate.grid(row=5, padx=10, pady=10)

    # some adjustments
    root.bind('<Return>', generate_pwd)
    entry_app_pwd.focus()

    root.mainloop()


def run_cli():
    # generate password by calling hash module
    # print password to shell
    def generate_pwd(*args):
        str_app_pwd = '123'
        str_master_pwd = '123'
        str_result = hash.run_hash(str_app_pwd, str_master_pwd)
        print(str_result)
        return ()

    generate_pwd()

    # TODO: write commandline mode

    return ()



# Hash ascii string with SHA-512
# Return: hex string of hash result
# @tested python3.5

import hashlib


def hash_sha512(str_byte_message):
    return hashlib.sha512(str_byte_message).hexdigest().lower()


# Hash package __init__.py
# Yu Gui    1/24/2017

from .sha512 import hash_sha512
from .expand import expand
from .transpose import transpose
from .base58_encode import base58_encode
from .trim_str import trim_str

import sys


# main hashing function
def run_hash(*args):
    # receive hashing scheme
    l_argv = sys.argv
    try:
        str_argv_scheme = l_argv[1]
    except IndexError:
        str_argv_scheme = "sha512-sha512-b58"

    l_argv_scheme = str_argv_scheme.split("-")
    l_argv_algo = l_argv_scheme[:-1]
    l_argv_encode = l_argv_scheme[-1:]
    print('Hashing algorithms: ', l_argv_algo)
    print('Encoding: ', l_argv_encode)

    str_seed1 = args[0]
    str_seed2 = args[1]

    str_input = str_seed1 + str_seed2
    str_result = str_input

    # parsing hash parameters
    for algo in l_argv_algo:
        # standard message digest algo(s)
        if algo == "sha512":
            str_input = str_result
            str_result = hash_sha512(str_input.encode())

            if __debug__:
                print('sha512: ', str_result)

        # transpose
        if algo == "t" or algo == "transpose":
            str_input = str_result
            str_result = transpose(str_input)

            if __debug__:
                print('transpose: ', str_result)

        # expansion
        try:
            if algo[0] == "e":
                i_target_len = int(algo[1:])
                str_input = str_result
                str_result = expand(str_input, i_target_len)

                if __debug__:
                    print('expansion: ', str_result)

        except IndexError:
            pass

    # define encode algorithms
    for encode_algo in l_argv_encode:
        # ASCII string encoding
        if encode_algo == "hstr":
            # do nothing
            pass

        # base 58 encoding
        if encode_algo == "b58":
            str_input = str_result
            str_result = base58_encode(str_input)

    # trim string to specified length
    str_result = trim_str(str_result, 12)

    print('Digest: ', str_result)
    print('Length: ', len(str_result))
    print('\n')

    return str_result



# Encode Hex string to Base 58 encoding
# Yu Gui    1/24/2017
# base 58 encoding is widely used in Bitcoin address encoding
# and is a very concise and unambiguous encoding procedure
# @tested python3.5


def base58_encode(str_input):
    __str_code = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
    i_code = int(str_input, 16)

    if __debug__:
        print('base 58 encoding: ')
        print('b16: ', str_input)
        print('b10: ', i_code)

    str_return = ""
    while i_code > 0:
        i_code, i_remainder = divmod(i_code, 58)
        str_return += __str_code[i_remainder]

    # reverse results of floor division to obtain final string
    str_temp = str_return[::-1]
    str_return = str_temp

    if __debug__:
        print('b58: ', str_return)

    return str_return



from distutils.core import setup

setup(
    name='IsoPass',
    version='0.1.1',
    author='Yu Gui',
    author_email='yugui923@gmail.com',
    packages=['isopass'],
    scripts=['isopass.py'],
    url='http://pypi.python.org/pypi/IsoPass/',
    license='GPL v3',
    description='Isolated Secure Offline Password Manager',
    long_description=open('README').read(),
    install_requires=[
    ],
)
